import React from 'react'
import classnames from 'classnames'
import { render, unmountComponentAtNode } from 'react-dom'

let instances_hide = [];

export default class Message extends React.Component {

    componentDidMount() {
        let hide = instances_hide.pop();
        hide && (this.timeid = hide());
    }

    renderDefault() {
        const {
            className, children, ...others
            } = this.props;

        const classes = classnames({
            ['Message--tips']: true,
            [className]: className,
        });

        return <div className={classes} {...others} ref={ref => this.target = ref}>
            <div className="Message--tips__text">
                { children || this.props.text }
            </div>
        </div>
    }

    renderFixedly() {
        const {
            className, children, ...others
            } = this.props;

        const classes = classnames({
            ['Message--tips-fixedly']: true,
            [className]: className,
        });

        return <div className={classes} {...others} ref={ref => this.target = ref}>
            <div className="Message--tips-fixedly__text">
                { children || this.props.text }
            </div>
        </div>
    }

    renderMovement() {
        const {
            className, children, ...others
            } = this.props;

        const classes = classnames({
            ['Message--tips-movement fadeInDown']: true,
            [className]: className,
        });

        return <div className={classes} {...others} ref={ref => this.target = ref}>
            <div className="Message--tips-movement__text">
                { children || this.props.text }
            </div>
        </div>
    }

    renderDebugger() {
        const {
            className, children, ...others
            } = this.props;

        const classes = classnames({
            ['Message--tips-debugger fadeInDown']: true,
            [className]: className,
        });

        return <div className={classes} {...others} ref={ref => this.target = ref}>
            <div className="Message--tips-debugger__text">
                { children || this.props.text }
            </div>
        </div>
    }

    render() {
        switch(this.props.type) {
            case 'fixedly':
                return this.renderFixedly();
            case 'movement':
                return this.renderMovement();
            case 'debugger':
                return this.renderDebugger();
            default:
                return this.renderDefault();
        }
    }
}

Message.propTypes = {
    className: React.PropTypes.string,
};

Message.show = (message, node) => {

    if (typeof message == 'string') {
        let duration = (typeof node === 'number') ? node : 3000
        message = <Message text={message} duration={duration}/>
    }

    let target = document.createElement('div');
    let parent = typeof node === 'object' ? node : document.body;

    if(message.props.type == 'debugger') {
        let debuggerContainer = document.getElementById('debuggerContainer');
        if(!debuggerContainer) {
            let container = document.createElement('div');
            container.id = 'debuggerContainer';
            container.style.cssText = 'position:fixed;left:10px;right:10px;top:0px;z-index:999';
            document.body.appendChild(container);
            parent = container;
        } else {
            parent = debuggerContainer;
        }
    }

    parent.insertBefore(target, parent.firstElementChild);

    let hide = () => {
        return setTimeout(() => {
            unmountComponentAtNode(target);
            parent.removeChild(target);
        }, parseInt(message.props.duration || 3000));
    }
    instances_hide.push(hide);

    render(message, target);
}
