import React, { Component } from 'react';

export default class Swipeout extends Component {

    static defaultProps = {
        btnWidth  : 70,    // 按钮宽度
        btnNum    : 2,
        index     : 0,
        classNames: '',
        hideCancelBtn: false,
        confirmText: '删除',
        cancelText: '取消',
        onDelOrder: () => {},
        onDelTransform: () => {}
    }

    static PropTypes = {
        btnWidth  : Number,
        btnNum    : Number,
        index     : Number,
        classNames: String,
        confirmText: String,
        hideCancelBtn: Boolean,
        cancelText: String,
        onDelOrder: Function,
        onDelTransform: Function,
    }

    constructor(props) {
        super(props);

        this.swipeInfo = {
            swipeWith: props.btnWidth * props.btnNum,
            startX   : 0,
            startY   : 0,
            direction: ''
        }
    }

    onSwipeStart = (e) => {
        const { onDelTransform, index } = this.props;

        onDelTransform(index);
        const touches = e.targetTouches && e.targetTouches[0] || e;
        this.swipeInfo.startX = touches.pageX;
        this.swipeInfo.startY = touches.pageY
    }

    onSwipeMove = (e) => {
        const changeTouches = e.targetTouches && e.targetTouches[0] || e;
        const disY = changeTouches.pageY - this.swipeInfo.startY;
        const disX = changeTouches.pageX - this.swipeInfo.startX;
        // 滑动偏上或者滑动偏下，就禁止左右滑动
        this.swipeInfo.direction = this.swipeDirection(disY, disX);
        const { direction, swipeWith } = this.swipeInfo;

        if (direction === 'left' || direction === 'right') {
            e.preventDefault();
            e.stopPropagation();
            
            const translateX = this.getTranslate();

            if (direction === 'left') {
                if (translateX && translateX[0] <= -swipeWith) return;

                const translateXDis = disX < -swipeWith ? -swipeWith : disX ;

                this.swipeTransition(translateXDis);
            } else {
                if (disX > swipeWith || !translateX || translateX[0] >= 0) return;
                
                const translateXDis = -swipeWith + disX > 0 ? 0 : -swipeWith + disX ;

                this.swipeTransition(translateXDis);
            }
        }
    }

    onSwipeEnd = (e) => {        
        const endTouches = e.changedTouches && e.changedTouches[0] || e;
        const offX = endTouches.pageX - this.swipeInfo.startX;
        const { direction, swipeWith } = this.swipeInfo;

        if (direction === 'left' && Math.abs(offX) > swipeWith / 3) {
            this.swipeTransition(-swipeWith);
        } else if (offX !== 0) {
            this.swipeTransition(0);
        }
    }

    onSwipeClick = (e) => {
        const translateX = this.getTranslate();

        if (translateX && translateX[0] != '0') {
            e.preventDefault();
            e.stopPropagation();
    
            this.swipeTransition(0);
        }
    }

    swipeTransition(distance = 0, time = '.2s', fun = 'linear') {
        const dom = this.refs.swipeItem;

        dom.style.webkitTransition = `all ${time} ${fun}`;
        dom.style.webkitTransform = `translate3d(${distance}px, 0, 0)`;
    }

    swipeDirection(disY, disX) {
        // 滑动角度
        const r = Math.atan2(disY, disX);
        let swipeAngle = Math.round(r * 180 / Math.PI);
        
        if (swipeAngle < 0) {
            swipeAngle = 360 - Math.abs(swipeAngle);
        }

        switch (true) {
            case (swipeAngle <= 210 && swipeAngle >= 150):
                return 'left';
            case (swipeAngle <= 30 || swipeAngle >= 330): 
                return 'right';
            case (swipeAngle > 210 && swipeAngle < 330): 
                return 'up';
            case (swipeAngle > 30 && swipeAngle < 150): 
                return 'down';
            default:
                return 'left';
        }
    }

    getTranslate() {
        return this.refs.swipeItem.style.webkitTransform.match(/\-?\d+\.?\d+/);
    }

    getSwipeEvents() {
        const swipeEvents = {
            onClick: this.onSwipeClick,
            onMouseDown: this.onSwipeStart,
            onMouseMove: this.onSwipeMove,
            onMouseUp: this.onSwipeEnd,
            onTouchStart: this.onSwipeStart,
            onTouchMove: this.onSwipeMove,
            onTouchEnd: this.onSwipeEnd
        }
        return swipeEvents
    }

    render() {
        const { 
            onDelOrder, 
            classNames, 
            btnWidth, 
            confirmText, 
            cancelText, 
            hideCancelBtn,
            children 
        } = this.props;

        return (
            <section className={`swipe-item ${classNames}`} { ...this.getSwipeEvents() }>
                <div className="swipe-item-ctn" ref="swipeItem" >
                    { children }
                </div>
                <div style={{width: `${btnWidth}px`}} className="swipe-item-btn confirm" onClick={onDelOrder}>{ confirmText }</div>
                {
                    !hideCancelBtn ?
                        <div style={{width: `${btnWidth}px`}} className="swipe-item-btn cancel">{ cancelText }</div> : null
                }
            </section>
        );
    }
};