```javascript

// MessageList.jsx

import {ScrollPaging} from 'vision-ui-ext';
import ChatActions from './ChatActions';

class MessageList extends Component {
  _loadMore() {
    this.props.dispatch(ChatActions.fetchMessages())
  }

  _renderMessages() {
    return _.map(this.props.messages, (msg) => {
      return(
          <div>{msg}</div>
      )
    })
  }

  render() {
    return (
        <ScrollPaging
          items={this._renderMessages()}
          loadMore={this._loadMore.bind(this)}
        />
    )
  }
}

```

Redux大概长这样

``` javascript
// ChatActions.js

export function fetchMessages(params) {
  return {
    type: FETCH_MESSAGES,
    messages: ['hi', 'hello', 'halo']
  };
}


// ChatReducer.js


const initialState = {
  messages: []
}

function chatReducer(state=initialState, action=undefined) {
  switch (action.type) {
    case FETCH_MESSAGES:
      return Object.assign({}, initialState, {
        messages: initialState.concat(action.messages)
      })
  }
}

```


### Options

| 属性  	|  类型 	|  必传 | 默认值 	| 描述  	|
|---	|---	|---	|---	|--- |
| threshold | integer | no | 100 | 距离容器底部的高度值
| hasMore  	|  bool 	| no  	| true  	| 是否有更多数据等待被展示. 可设置为false一旦所有 `items`或`children`展示完全.
| loadingMore |  bool 	| no  	|false| 是否正在加载，items加载完设置false
| loader  	|  any | no|  Loading... 	| `hasMore` `loadingMore` `showLoader` 全部为`true`时，会显示这个加载画面。
| showLoader  	| bool  	| false  	|  true 	| 是否要显示加载画面
| loadMore  	|  function | yes  |undefined| 当需要加载时会触发该函数.
| holderType | string  	|no|div| 加载列表标签类型，如`ul`, `dl`.
| className | string  	|no|''| 加载列表的className.
| items | array || Immutable List 	|no|[]| 等待渲染的元素列表.
| children | array || Immutable List 	|no| [] | 等待渲染的元素列表.
