"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.shallowEqual = shallowEqual;

function shallowEqual(prev, next) {
    if (prev === next) return true;
    var prevKeys = Object.keys(prev);
    var nextKeys = Object.keys(next);

    if (prevKeys !== nextKeys) return false;

    return prevKeys.every(function (key) {
        return prevKeys.hasOwnProperty(key) && prevKeys[key] === nextKeys[key];
    });
}

function PureRender(Component) {
    Component.prototype.shouldComponentUpdate = function (nextProps, nextState) {
        return PureRender.shouldComponentUpdate(nextProps, nextState, this.props, this.state);
    };
}

PureRender.shouldComponentUpdate = function (nextProps, nextState, preProps, preState) {
    return !shallowEqual(preProps, nextProps) || !shallowEqual(preState, nextState);
};

exports["default"] = PureRender;